/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.tizenopensource.org/license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#ifndef __MAIN_H__
#define __MAIN_H__

#include <Elementary.h>
#include <appcore-efl.h>
#include <Ecore_X.h>
#include <glib.h>
#include <glib/gstdio.h>
#include <glib-object.h>
#include <glib/gprintf.h>

#if !defined(PACKAGE)
#  define PACKAGE "w-launcher-app"
#endif

#if !defined(RESDIR)
#  define RESDIR "/usr/apps/org.tizen.w-launcher-app/res"
#endif

#if !defined(LOCALEDIR)
#  define LOCALEDIR RESDIR"/locale"
#endif

#if !defined(EDJDIR)
#  define EDJDIR RESDIR"/edje"
#endif

#if !defined(PKGNAME)
#  define PKGNAME "org.tizen.w-launcher-app"
#endif

#define EDJ_APP EDJDIR"/w_launcher_app.edj"

#define S_(str) dgettext("sys_string", str)
#define T_(str) dgettext(PACKAGE, str)
#undef N_
#define N_(str) (str)
#undef _
#define _(str) gettext(str)

#define _EDJ(x) elm_layout_edje_get(x)
#define _X(x) (x*elm_config_scale_get())

#define WIN_SIZE 320
#define MAX_APP_NUM 128

#define SERVICE_OPERATION_MAIN "http://tizen.org/appcontrol/operation/main"

#define USE_SETTING_IDLE_CLOCK 1
#define USE_AUL_LAUNCH 0

#define WAKEUP_GESTURE_DBUS_PATH "/org/tizen/sensor/context/gesture/wakeup"
#define WAKEUP_GESTURE_DBUS_INTERFACE "org.tizen.sensor.context.gesture.wakeup"
#define WAKEUP_GESTURE_DBUS_MEMBER_CLOCK "Clock"

#define APP_NAME_BT_POPUP "org.tizen.bt-connection-popup"

#define APP_TYPE_WIDGET "wgt"
#define APP_TYPE_RPM "rpm"
#define APP_TYPE_TPK "tpk"

enum {
	POS_DOWN = 0,
	POS_MOVE,
	POS_UP,
	POS_REC,//to record the pre positionbf
	STEP,
	POS_MAX
};

enum {
	APP_STATE_PAUSE = 1,
	APP_STATE_SERVICE,
	APP_STATE_RESUME,
	APP_STATE_TERMINATE,
};

typedef enum {
	W_LAUNCHER_ERROR_OK = 0,
	W_LAUNCHER_ERROR_FAIL = -1,
	W_LAUNCHER_ERROR_DB_FAILED = -2,
	W_LAUNCHER_ERROR_OUT_OF_MEMORY = -3,
	W_LAUNCHER_ERROR_INVALID_PARAMETER = -4,
	W_LAUNCHER_ERROR_NO_DATA = -5,
} w_launcher_error_e;

typedef struct favorite_apps_info {
	char *pkgname;
	char *classname;
	int order;
} favorite_apps_info;

typedef struct menu_apps_info {
	char *app_id;
	char *app_name;
	char *icon_path;
	char *type;
} more_apps_info;

struct appdata
{
	Evas_Object *win;
	Evas_Object *ly_main;
	int id_launch_effect;

	Evas_Object *win_apps;
	Evas_Object *ly_apps;

	Ecore_X_Window input_win;

	int win_w;
	int win_h;

	int app_state;
	int count_grabed;
	int powerkey_count;
	Eina_Bool need_to_lcdon;

	Evas_Object *scroller;
	Evas_Object *main_box;
	Evas_Object *mapbuf[14];
	Evas_Object *page_layout[14];

	int current_page;
	int total_page;
	int idle_clock_page;

	Evas_Object *scroller_apps;
	Evas_Object *menuapps_box;
	Evas_Object *mapbuf_apps[10];
	Evas_Object *page_layout_apps[10];

	int current_page_apps;
	int total_page_apps;

	Eina_Bool flag_mouse_down;
	Eina_Bool flag_long_press;
	Eina_Bool powerkey_lcd_state;
	Evas_Coord posx[POS_MAX];
	Evas_Coord posy[POS_MAX];

	Elm_Theme *theme;

	Ecore_Timer *timer;
	Ecore_Timer *powerkey_timer;
	Ecore_Timer *long_press_timer;
	Ecore_Event_Handler *key_down_handler;
	Ecore_Event_Handler *key_up_handler;

	Eina_Bool flag_wakeup_setting_clock;
	Eina_Bool flag_launching_more_apps;
	Ecore_Timer *moreapps_timer;

	/* relaunch flag */
	Eina_Bool flag_need_to_relaunch;

	/* more apps */
	GArray *more_apps_list;

	/* minicontrol */
	Evas_Object *minictl_idle_clock;
	char *pkgname_idle_clock;
	int pid_idle_clock;
	char *appid_idle_clock;

	/* favorite apps */
	GArray *favorite_apps_list;

	/* DBUS */
	DBusConnection *connection;
};

struct appdata *w_launcher_app_data_get(void);

#endif /* __MAIN_H__ */

