/*
 * Copyright (c) 2010 - 2011 Samsung Electronics Co., Ltd. All rights reserved.
 *
 * PROPRIETARY/CONFIDENTIAL
 *
 * This software is the confidential and proprietary information of
 * SAMSUNG ELECTRONICS ("Confidential Information"). You agree and acknowledge
 * that this software is owned by Samsung and you shall not disclose
 * such Confidential Information and shall use it only in accordance with the
 * terms of the license agreement you entered into with SAMSUNG ELECTRONICS.
 * SAMSUNG make no representations or warranties about the suitability
 * of the software, either express or implied, including but not limited
 * to the implied warranties of merchantability, fitness for a particular purpose,
 * or non-infringement. SAMSUNG shall not be liable for any damages suffered
 * by licensee arising out of or releated to this software.
 *
 */

#include <appcore-common.h>
#include <vconf.h>
#include <vconf-keys.h>
#include <dlog.h>
#include <ctype.h>

#include <unicode/utypes.h>
#include <unicode/putil.h>
#include <unicode/uiter.h>
#include <unicode/udat.h>
#include <unicode/udatpg.h>
#include <unicode/ustring.h>

#include "main.h"
#include "log.h"
#include "logic.h"

#define BUFFER_LENGTH 256

static bool get_formatted_ampm_from_utc_time(UDate date, char* date_str, int date_size, int* str_length, Eina_Bool* is_pre, const char *timezone)
{
	UChar customSkeleton[BUFFER_LENGTH] = { 0 };
	UErrorCode status = U_ZERO_ERROR;
	UDateFormat *formatter = NULL;

	UChar bestPattern[BUFFER_LENGTH] = { 0 };
	UChar formatted[BUFFER_LENGTH] = { 0 };

	char bestPatternString[BUFFER_LENGTH] = { 0 };
	char formattedString[BUFFER_LENGTH] = { 0 };

	UDateTimePatternGenerator *pattern_generator = NULL;

	char *time_skeleton = "hhmm";

	char *locale = vconf_get_str(VCONFKEY_REGIONFORMAT);
	if (locale == NULL) {
		_E("[Error] get value of VCONFKEY_REGIONFORMAT fail.");
		return false;
	}

	u_uastrncpy(customSkeleton, time_skeleton, strlen(time_skeleton));

	pattern_generator = udatpg_open(locale, &status);

	int32_t bestPatternCapacity = (int32_t) (sizeof(bestPattern) / sizeof((bestPattern)[0]));
	(void)udatpg_getBestPattern(pattern_generator, customSkeleton,
		u_strlen(customSkeleton), bestPattern,
			bestPatternCapacity, &status);

	u_austrcpy(bestPatternString, bestPattern);
	u_uastrcpy(bestPattern,"a");

	if(bestPatternString[0] == 'a')
	{
		(*is_pre) = EINA_TRUE;
	}
	else
	{
		(*is_pre) = EINA_FALSE;
	}

	UChar timezoneID[64] = {0,};
	if(timezone != NULL) {
		u_uastrncpy(timezoneID, timezone, 64);
		formatter = udat_open(UDAT_IGNORE, UDAT_IGNORE, locale, timezoneID, -1, bestPattern, -1, &status);
	} else {
		formatter = udat_open(UDAT_IGNORE, UDAT_IGNORE, locale, NULL, -1, bestPattern, -1, &status);
	}

	int32_t formattedCapacity = (int32_t) (sizeof(formatted) / sizeof((formatted)[0]));
	(void)udat_format(formatter, date, formatted, formattedCapacity, NULL, &status);
	u_austrcpy(formattedString, formatted);

	_D("DATE & TIME is %s %s %d %s", locale, formattedString, u_strlen(formatted), bestPatternString);

	(*str_length) = u_strlen(formatted);

	udatpg_close(pattern_generator);

	udat_close(formatter);

	if(strlen(formattedString) < date_size)	{
		strncpy(date_str, formattedString, strlen(formattedString));
	} else {
		strncpy(date_str, formattedString, date_size - 1);
	}

	if(locale) {
		free(locale);
	}

	return true;
}

static char *_get_locale(void)
{
	char *locale = vconf_get_str(VCONFKEY_REGIONFORMAT);
	if(locale == NULL) {
		_E("vconf_get_str() failed : region format");
		return strdup("en_US.UTF-8");
	}
	return locale;
}

static UDateFormat *_get_formatter_date(void *data, const char *timezone, Eina_Bool is_info, const char *skeleton)
{
	UErrorCode status = U_ZERO_ERROR;
	char *locale = NULL;
	UChar u_timezone[64] = {0,};
	UChar u_skeleton[64] = {0,};
	int skeleton_len = 0;

	UDateFormat *formatter = NULL;
	UDateTimePatternGenerator *generator = NULL;

	UChar u_best_pattern[64] = {0,};
	int32_t u_best_pattern_capacity;
	int u_best_pattern_len = 0;

	struct appdata *ad = data;
	retv_if(ad == NULL, NULL);

#ifdef HAVE___SECURE_GETENV
	uloc_setDefault(__secure_getenv("LC_TIME"), &status);/* for thread saftey  */
#else
	uloc_setDefault(getenv("LC_TIME"), &status);/* for thread saftey  */
#endif
	if (U_FAILURE(status)) {
		_E("uloc_setDefault() is failed.");
		return NULL;
	}

	locale = _get_locale();

	u_uastrncpy(u_skeleton, skeleton, strlen(skeleton));
	skeleton_len = u_strlen(u_skeleton);

	generator = udatpg_open(locale, &status);
	if(U_FAILURE(status)) {
		_E("udatpg_open() failed");
		generator = NULL;
		if(locale) {
			free(locale);
			locale = NULL;
		}
		return NULL;
	}

	u_best_pattern_capacity = (int32_t) (sizeof(u_best_pattern) / sizeof((u_best_pattern)[0]));

	u_best_pattern_len = udatpg_getBestPattern(generator, u_skeleton, skeleton_len, u_best_pattern, u_best_pattern_capacity, &status);
	if(U_FAILURE(status)) {
		_E("udatpg_getBestPattern() failed");
		if(locale) {
			free(locale);
			locale = NULL;
		}
	}

	if(timezone) {
		u_uastrncpy(u_timezone, timezone, sizeof(u_timezone));
		formatter = udat_open(UDAT_IGNORE, UDAT_DEFAULT, locale, u_timezone, -1,
								u_best_pattern, u_strlen(u_best_pattern), &status);
	} else {
		formatter = udat_open(UDAT_IGNORE, UDAT_DEFAULT, locale, NULL, -1,
								u_best_pattern, u_strlen(u_best_pattern), &status);
	}
	if(U_FAILURE(status)) {
		_E("udat_open() is failed.");
		if(locale) {
			free(locale);
			locale = NULL;
		}
		return NULL;
	}

	if(locale) {
		free(locale);
		locale = NULL;
	}

	if(generator) {
		udat_close(generator);
		generator = NULL;
	}

	return formatter;
}

int get_formatted_date_from_utc_time(time_t intime, char *buf, int buf_len, const char *timezone, void *data, const char *format)
{
	struct appdata *ad = data;
	retv_if(ad == NULL, -1);

	int formatted_str_len = -1;
	UDate u_time = (UDate)intime *1000;
	UChar u_formatted_str[64] = {0,};
	int32_t u_formatted_str_capacity;
	UErrorCode status = U_ZERO_ERROR;

	UDateFormat *formatter = NULL;

	if(format) {
		formatter = _get_formatter_date(ad, timezone, EINA_TRUE, format);
	} else {
		formatter = _get_formatter_date(ad, timezone, EINA_TRUE, "MMMEd");
	}
	if(formatter == NULL) {
		_E("_get_formatter_time() failed");
		return -1;
	}

	u_formatted_str_capacity = (int32_t)(sizeof(u_formatted_str) / sizeof((u_formatted_str)[0]));

	formatted_str_len = udat_format(formatter, u_time, u_formatted_str, u_formatted_str_capacity, NULL, &status);
	if(U_FAILURE(status)) {
		_E("udat_format() failed");
		if(formatter) {
			udat_close(formatter);
			formatter = NULL;
		}
		return -1;
	}

	buf = u_austrncpy(buf, u_formatted_str, buf_len);
	_SECURE_D("time : %s %d", buf, intime);
	if(formatter) {
		udat_close(formatter);
		formatter = NULL;
	}
	return 0;
}

static Eina_Bool _set_info_time(void *data)
{
	struct appdata *ad = data;
	if (ad == NULL) {
		_D("appdata is NULL");
		return false;
	}

	struct tm st;
	time_t tt;
	char buf[512] = { 0, };
	char bf1[32] = { 0, };
	char bf2[32] = { 0, };
	int r, hour;
	enum appcore_time_format timeformat;

	tt = time(NULL);
	localtime_r(&tt, &st);

	if (ad->timer != NULL) {
		ecore_timer_del(ad->timer);
		ad->timer = NULL;
	}

	char utc_date[256] = { 0, }; //txt.date

	/* txt.date */
	get_formatted_date_from_utc_time(tt, utc_date, sizeof(utc_date), NULL, ad, UDAT_ABBR_MONTH_WEEKDAY_DAY);
	elm_object_part_text_set(ad->page_layout[ad->idle_clock_page], "txt.date", utc_date);

	int val;
	vconf_get_int(VCONFKEY_PM_STATE, &val);
	if(val != VCONFKEY_PM_STATE_LCDOFF) {
		ad->timer = ecore_timer_add(60 - st.tm_sec, _set_info_time, ad);
	}

	r = appcore_get_timeformat(&timeformat);

	/* if %l is 1 ~9, it has blank space in front. Trim it using atoi function */
	strftime(bf1, sizeof(bf1), "%H", &st);
	hour = atoi(bf1);
	strftime(bf1, sizeof(bf1), ":%M", &st);
	snprintf(buf, sizeof(buf), "%d%s", hour, bf1);
	//snprintf(buf, sizeof(buf), "%02d%s", hour, bf1);

	char utc_ampm[BUFFER_LENGTH] = { 0 };
	int ampm_length = 0;
	Eina_Bool is_pre = EINA_FALSE;

	get_formatted_ampm_from_utc_time(ucal_getNow(), utc_ampm, sizeof(utc_ampm), &ampm_length, &is_pre, NULL);
	if(ampm_length > 0 && ampm_length <= 4)	{
		snprintf(bf2, sizeof(bf2), "%s", utc_ampm);
	} else {
		if (st.tm_hour >= 0 && st.tm_hour < 12) {
			snprintf(bf2, sizeof(bf2), "%s", "AM");
		} else {
			snprintf(bf2, sizeof(bf2), "%s", "PM");
		}
	}
	elm_object_part_text_set(ad->page_layout[ad->idle_clock_page], "txt.clock", buf);

	return ECORE_CALLBACK_RENEW;
}

static void _time_changed_cb(keynode_t *node, void *data)
{
	_D("%s", __func__);
	struct appdata *ad = data;
	ret_if(ad == NULL);

	_set_info_time(ad);
}

void set_info(void *data)
{
	_D("%s", __func__);
	struct appdata *ad = data;
	ret_if(ad == NULL);

	int ret = -1;

	/* register vconf changed cb */
	ret = vconf_notify_key_changed(VCONFKEY_SYSTEM_TIME_CHANGED, _time_changed_cb, ad);
	if(ret > 0) {
		_D("Failed to register vconfkey changed cb.");
	}

	_set_info_time(ad);
}

void update_time(void *data)
{
	_D("%s", __func__);
	struct appdata *ad = data;
	ret_if(ad == NULL);

	_set_info_time(ad);
}

void pause_time(void *data)
{
	_D("%s", __func__);
	struct appdata *ad = data;
	ret_if(ad == NULL);

	if(ad->timer) {
		ecore_timer_del(ad->timer);
		ad->timer = NULL;
		_D("Delete ad->timer");
	}
}
