/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.tizenopensource.org/license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Ecore.h>
#include <Ecore_X.h>
#include <utilX.h>
#include <vconf.h>
#include <vconf-keys.h>
#include <aul.h>
#include <minicontrol-viewer.h>
#include <minicontrol-monitor.h>

#include "main.h"
#include "log.h"
#include "scroller.h"
#include "logic.h"
#include "util_efl.h"
#include "clock.h"
#include "pkgmgr.h"
#include "app_tray.h"
#include "dbus.h"

/* App Icon */
#define IMGDIR "/usr/apps/org.tizen."PACKAGE"/res/images/w-launcher-app"
#define ICON_PATH_NOTIFICATION IMGDIR"/b_home_screen_notification.png"
#define ICON_PATH_LOGS IMGDIR"/b_home_screen_call_log.png"
#define ICON_PATH_CONTACTS IMGDIR"/b_home_screen_contact.png"
#define ICON_PATH_MORE_APPS IMGDIR"/b_home_screen_dialer_more_apps.png"

/* packages */
#define PKG_NAME_NOTIFICATION "org.tizen.wnotification"
#define PKG_NAME_LOGS "org.tizen.logs"
#define PKG_NAME_CONTACTS "org.tizen.contacts"

/* DATA KEY */
#define DATA_KEY_ANIM_START "anim_start"
#define DATA_KEY_ANIM_START_X "anim_start_x"
#define DATA_KEY_DRAG_START "drag_start"
#define DATA_KEY_SCROLL_BY_USER "scroll_by_user"
#define DATA_KEY_SCROLLING_PAGE "scrolling_page"
#define DATA_KEY_PREVIOUS_X "previous_x"
#define DATA_KEY_CURRENT_IDX "current_idx"

#define PAGE_MOVE 1
#define PAGE_APP_NUM 4
#define PAGE_MAX_NUM 10

#define WNOTI_COUNT "file/private/home/noti_count"

#define USE_MAPBUF 0

#ifndef USE_SETTING_IDLE_CLOCK
/* Weather Icon */
#define WEATHER_ICON_CLEAR IMGDIR"/weather/clear.png"
#define WEATHER_ICON_COLD IMGDIR"/weather/cold.png"
#define WEATHER_ICON_FLURRIES IMGDIR"/weather/flurries.png"
#define WEATHER_ICON_FOG IMGDIR"/weather/fog.png"
#define WEATHER_ICON_HAIL IMGDIR"/weather/hail.png"
#define WEATHER_ICON_HEAVY_RAIN IMGDIR"/weather/heavy_rain.png"
#define WEATHER_ICON_HOT IMGDIR"/weather/hot.png"
#define WEATHER_ICON_ICE IMGDIR"/weather/ice.png"
#define WEATHER_ICON_MOSTLY_CLEAR IMGDIR"/weather/mostly_clear.png"
#define WEATHER_ICON_MOSTLY_CLOUD IMGDIR"/weather/mostly_cloud.png"
#define WEATHER_ICON_MOSTLY_CLOUDY_FLURRIES IMGDIR"/weather/mostly_cloudy_flurries.png"
#define WEATHER_ICON_MOSTLY_CLOUDY_THUNDER_SHOWERS IMGDIR"/weather/mostly_cloudy_thunder_showers.png"
#define WEATHER_ICON_PARTLY_SUNNY IMGDIR"/weather/partly_sunny.png"
#define WEATHER_ICON_PARTLY_SUNNY_SHOWERS IMGDIR"/weather/partly_sunny_showers.png"
#define WEATHER_ICON_RAIN IMGDIR"/weather/rain.png"
#define WEATHER_ICON_RAIN_SNOW IMGDIR"/weather/rain_snow.png"
#define WEATHER_ICON_SANDSTORM IMGDIR"/weather/sandstorm.png"
#define WEATHER_ICON_SHOWERS IMGDIR"/weather/showers.png"
#define WEATHER_ICON_SNOW IMGDIR"/weather/snow.png"
#define WEATHER_ICON_SUNNY IMGDIR"/weather/sunny.png"
#define WEATHER_ICON_THUNDERSTORMS IMGDIR"/weather/thunderstorms.png"
#define WEATHER_ICON_WINDY IMGDIR"/weather/windy.png"

#define TEMP_UNIT_C "c"
#define TEMP_UNIT_F "f"

const char *weather_icon_path[] = {
	WEATHER_ICON_SUNNY, /* undefined : 0 */
	WEATHER_ICON_SUNNY, /* weather id : 1 */
	WEATHER_ICON_PARTLY_SUNNY, /* 2 */
	WEATHER_ICON_MOSTLY_CLOUD, /* 3 */
	WEATHER_ICON_SHOWERS, /* 4 */
	WEATHER_ICON_FOG, /* 5 */
	WEATHER_ICON_PARTLY_SUNNY_SHOWERS, /* 6 */
	WEATHER_ICON_THUNDERSTORMS, /* 7 */
	WEATHER_ICON_MOSTLY_CLOUDY_THUNDER_SHOWERS, /* 8 */
	WEATHER_ICON_RAIN, /* 9 */
	WEATHER_ICON_FLURRIES, /* 10 */
	WEATHER_ICON_MOSTLY_CLOUDY_FLURRIES, /* 11 */
	WEATHER_ICON_SNOW, /* 12 */
	WEATHER_ICON_RAIN_SNOW, /* 13 */
	WEATHER_ICON_WINDY, /* 14 */
	WEATHER_ICON_ICE, /* 15 */
	WEATHER_ICON_HOT, /* 16 */
	WEATHER_ICON_COLD, /* 17 */
	WEATHER_ICON_CLEAR, /* 18 */
	WEATHER_ICON_MOSTLY_CLEAR /* 19 */
};
#endif

#define E_DATA_MINICONTROL_TAG "tag"

#if 0
inline void index_bring_in(Evas_Object *index, int idx)
{
	ret_if(NULL == index);
	_D(" >>>>> idx : %d", idx);

	/* If index is updated, then it is not selected basically. */
	/* So, do not try this : if (cur_idx == idx) return; */

	int cur_idx = (int) evas_object_data_get(index, DATA_KEY_CURRENT_IDX);
	if (idx == cur_idx) return;
	evas_object_data_set(index, DATA_KEY_CURRENT_IDX, (void *) idx);

	Elm_Object_Item *idx_it = elm_index_item_find(index, (void *) idx);
	if (NULL == idx_it) return;
	elm_index_item_selected_set(idx_it, EINA_TRUE);

	return;
}
#endif

static struct {
	int favorite_pid;

	Ecore_Timer *select_item_timer;
} home_event_info = {
	.favorite_pid = 0,

	.select_item_timer = NULL,
};

static void _favorite_apps_selected_timer_cb(void *data)
{
	_D("%s : %s", __func__, data);

	home_event_info.favorite_pid = 0;
	_D("set home_event_info.favorite_pid : 0");

	if(home_event_info.select_item_timer) {
		ecore_timer_del(home_event_info.select_item_timer);
		home_event_info.select_item_timer = NULL;
		_D("delete home_event_info.select_item_timer");
	}
}

static void _launch_app_cb(void *data, Evas_Object *o, const char *emission, const char *source)
{
	_D("%s : %s", __func__, data);

	/* check App launching */
	if(home_event_info.favorite_pid != 0) {
		_D("App is launching");
		return;
	}

#if USE_AUL_LAUNCH
	bundle *b = NULL;
	//int pid = 0;
	b = bundle_create();

	if(!b) {
		_E("Failed to create bundle");
		return;
	}

	bundle_add(b, "__APP_SVC_OP_TYPE__", SERVICE_OPERATION_MAIN);

	home_event_info.favorite_pid = aul_launch_app(data, b);
	if(home_event_info.favorite_pid < AUL_R_OK) {
		_E("Launching app ret : [%d]", home_event_info.favorite_pid);
		home_event_info.favorite_pid = 0;
		if(b) {
			bundle_free(b);
			_D("bundle free");
		}
		return;
	} else {
		_D("add item selected timer");
		ecore_timer_add(1.0, _favorite_apps_selected_timer_cb, data);
	}

	if(b) {
		bundle_free(b);
		_D("bundle free");
	}

#else
	int ret_aul = -1;

	ret_aul = aul_open_app(data);
	if(ret_aul < AUL_R_OK) {
		_E("Launching app ret : [%d]", ret_aul);
		return;
	}
	if(home_event_info.favorite_pid < AUL_R_OK) {
		_E("Launching app ret : [%d]", home_event_info.favorite_pid);
		home_event_info.favorite_pid = 0;
		return;
	} else {
		_D("add item selected timer");
		ecore_timer_add(1.0, _favorite_apps_selected_timer_cb, data);
	}
#endif
}

#if 0
static void _ly_apps_mouse_down_cb(void *data, Evas * evas, Evas_Object * obj, void *event_info)
{
	struct appdata *ad = data;
	ret_if(ad == NULL);

	Evas_Event_Mouse_Down *ev = event_info;

	ad->flag_mouse_down = EINA_TRUE;

	ad->posx[POS_DOWN] = ev->output.x;
	ad->posy[POS_DOWN] = ev->output.y;
}

static void _ly_apps_mouse_move_cb(void *data, Evas * evas, Evas_Object * obj, void *event_info)
{
	struct appdata *ad = data;
	ret_if(ad == NULL);

	Ecore_Event_Mouse_Move *ev = event_info;
	_D(" >>>> down(%d) - move(%d) = %d ", ad->posy[POS_DOWN],  ev->y, ad->posy[POS_DOWN] - ev->y);

	if(ad->flag_mouse_down) {
		if(ad->posy[POS_DOWN] - ev->y < -100) {
			_D("BACK!!");
#if 0
			evas_object_del(ad->win_apps);
			ad->win_apps = NULL;
			return;
#endif
		}
	}
}

static void _ly_apps_mouse_up_cb(void *data, Evas * evas, Evas_Object * obj, void *event_info)
{
	struct appdata *ad = data;
	ret_if(ad == NULL);

	//Evas_Event_Mouse_Up *ev = event_info;

	ad->flag_mouse_down = EINA_FALSE;
}
#endif

int page_scroller_get_current_page_no(Evas_Object *scroller)
{
	_D("%s", __func__);
	return (int) evas_object_data_get(scroller, DATA_KEY_CURRENT_IDX);
}

#if 0
void page_scroller_bring_in(Evas_Object *scroller, int idx)
{
	_D("Scroller(%p) brings into the (%d) page", scroller, idx);

	//int count = page_scroller_count_page(scroller);
	int count = 5;
	if (idx < 0) return;

	int origin_idx = (int) evas_object_data_get(scroller, DATA_KEY_CURRENT_IDX);
	if (idx == count) {
		_D("Looping");
		evas_object_data_set(scroller, DATA_KEY_CURRENT_IDX, (void *) 0);
		return;
	}
	evas_object_data_set(scroller, DATA_KEY_CURRENT_IDX, (void *) idx);

	int x, y, w, h;
	evas_object_geometry_get(scroller, &x, &y, &w, &h);
	if (w && (0 == x % w) && (idx == origin_idx)) {
		_D("No need to bring in the scroller : (x:%d, w:%d) (idx:%d, origin_idx:%d)", x, w, idx, origin_idx);
		return;
	}

	//elm_scroller_region_bring_in(scroller, idx * w, 0, w, h);
	elm_scroller_page_bring_in(scroller, idx, 0);
}
#endif

#if 0
static void _scroller_anim_start_cb(void *data, Evas_Object *scroller, void *event_info)
{
	_D("%s", __func__);

	Evas_Coord x, y, w, h;
	elm_scroller_region_get(scroller, &x, &y, &w, &h);

	evas_object_data_set(scroller, DATA_KEY_ANIM_START, (void *) 1);
	evas_object_data_set(scroller, DATA_KEY_ANIM_START_X, (void *) x);

	int drag_start = (int) evas_object_data_get(scroller, DATA_KEY_DRAG_START);
	if (drag_start == 0) return;
	evas_object_data_set(scroller, DATA_KEY_DRAG_START, (void *) 0);
}
#endif

static void _scroller_anim_stop_cb(void *data, Evas_Object *scroller, void *event_info)
{
	_D("%s", __func__);

	struct appdata *ad = (struct appdata *)data;
	ret_if(ad == NULL);

	int idx = -1;

	Evas_Coord x, y, w, h;
	elm_scroller_region_get(ad->scroller, &x, &y, &w, &h);

	idx = x / w;

	if(idx == ad->idle_clock_page) {
		/* set LCD off  - 0 : enable lcd off by power key */
		//send_lcd_off_signal(0);

	} else {
		/* set LCD on  - 1 : disable lcd off by power key */
		//send_lcd_off_signal(1);

	}
}

#if 0
static void _scroller_drag_start_cb(void *data, Evas_Object *scroller, void *event_info)
{
	_D("%s", __func__);

	int previous_x;

	elm_scroller_region_get(scroller, &previous_x, NULL, NULL, NULL);
	evas_object_data_set(scroller, DATA_KEY_PREVIOUS_X, (void *) previous_x);
	evas_object_data_set(scroller, DATA_KEY_DRAG_START, (void *) 1);
	evas_object_data_set(scroller, DATA_KEY_SCROLL_BY_USER, (void *) 1);
}
#endif

#if 0
static void _scroller_drag_stop_cb(void *data, Evas_Object *scroller, void *event_info)
{
	_D("%s", __func__);

	Evas_Coord x;
	Evas_Coord w;
	Evas_Coord y;
	Evas_Coord h;
	int previous_x;

	elm_scroller_region_get(scroller, &x, &y, &w, &h);
	previous_x = (int) evas_object_data_get(scroller, DATA_KEY_PREVIOUS_X);
	if (x == previous_x) {
		_D("Hold scroller, previous_x(%d) current(%d)",	previous_x, x);
	}

	/* _anim_stop_cb */
	do {
		if (x % w) return;
		evas_object_data_set(scroller, DATA_KEY_SCROLLING_PAGE, (void *) 0);

		int idx = 0;
		if (w) idx = x / w;

		page_scroller_bring_in(scroller, idx);
	} while (0);
}
#endif

static void _box_layout_cb(Evas_Object *o, Evas_Object_Box_Data *priv, void *data)
{
	_D("%s", __func__);
	struct appdata *ad = (struct appdata *)data;
	ret_if(ad == NULL);

	Evas_Coord x=0, y=0, w=0, h=0;
	int size_w = 0;
	int n_children = 0;
	Evas_Object_Box_Option *opt = NULL;
	Evas_Coord child_w=0, child_h=0;
	Eina_List *l = NULL;
	Eina_List *l_next = NULL;

	evas_object_geometry_get(o, &x, &y, &w, &h);

	n_children = eina_list_count(priv->children);
	if(n_children == 0)
		return;

	opt = eina_list_data_get(priv->children);
	if(opt == NULL)
		return;
	if(opt->obj == NULL)
		return;
	evas_object_size_hint_min_get(opt->obj, &child_w, &child_h);

	int order_children = 0;
	EINA_LIST_FOREACH_SAFE(priv->children, l, l_next, opt)
	{
		evas_object_move(opt->obj, x + WIN_SIZE * order_children, y);
		order_children++;
	}

	size_w = WIN_SIZE * order_children;
	evas_object_size_hint_min_set(o, size_w, -1);
	ad->total_page = order_children;
	_D("box total size : %d * %d = %d", WIN_SIZE, order_children, size_w);
}

#ifndef USE_SETTING_IDLE_CLOCK
static void _temp_changed_cb(keynode_t * node, void *data)
{
	_D("%s", __func__);
	struct appdata *ad = (struct appdata *)data;
	ret_if(ad == NULL);

	int ret = -1;
	int temp = -1;
	char buf[128] = { 0, };

	/* weather - temp */
	ret = vconf_get_int(VCONFKEY_DAILYBRIEFING_WEATHER_TEMP, &temp);
	if(ret < 0) {
		_D("Failed to get temp");
		return;
	}
	_D("temperature = %d", temp);
	snprintf(buf, sizeof(buf), "%d", temp);
	elm_object_part_text_set(ad->page_layout[ad->idle_clock_page], "txt.temp", buf);
}

static void _temp_unit_changed_cb(keynode_t * node, void *data)
{
	_D("%s", __func__);
	struct appdata *ad = (struct appdata *)data;
	ret_if(ad == NULL);

	char *temp_unit = NULL;

	/* weather - celsius/fahrenheit */
	temp_unit = vconf_get_str(VCONFKEY_DAILYBRIEFING_WEATHER_TEMP_UNIT);
	if(temp_unit == NULL || strlen(temp_unit) < 1){
		_D("temp unit is NULL : use celsius");
		elm_object_signal_emit(ad->page_layout[ad->idle_clock_page], "change.weather.temp.celsius", "img.temp.unit");
		return;
	}else {
		_D("unit = %s", temp_unit);
		if(!strncmp(temp_unit, TEMP_UNIT_C, sizeof(TEMP_UNIT_C))) {
			elm_object_signal_emit(ad->page_layout[ad->idle_clock_page], "change.weather.temp.celsius", "img.temp.unit");
		} else if(!strncmp(temp_unit, TEMP_UNIT_F, sizeof(TEMP_UNIT_F))) {
			elm_object_signal_emit(ad->page_layout[ad->idle_clock_page], "change.weather.temp.fahrenheit", "img.temp.unit");
		}
		free(temp_unit);
		temp_unit = NULL;
	}
}

static void _temp_icon_changed_cb(keynode_t * node, void *data)
{
	_D("%s", __func__);
	struct appdata *ad = (struct appdata *)data;
	ret_if(ad == NULL);

	int ret = -1;
	int weather_id = -1;
	Evas_Object *icon = NULL;

	/* weather - icon */
	ret = vconf_get_int(VCONFKEY_DAILYBRIEFING_WEATHER_ICON_ID, &weather_id);
	_D("weather icon id = %d", weather_id);
	if(ret < 0) {
		_D("weather icon id is NULL");
		icon = make_icon(ad, ad->ly_main, WEATHER_ICON_SUNNY);
		elm_object_part_content_set(ad->page_layout[ad->idle_clock_page], "sw.weather.icon", icon);
		return;
	} else {
		icon = make_icon(ad, ad->ly_main, weather_icon_path[weather_id]);
		elm_object_part_content_set(ad->page_layout[ad->idle_clock_page], "sw.weather.icon", icon);
	}
}
#endif

static int _get_length_of_count(int data)
{
	_D("%s", __func__);
	int len = 1, i;

	if (data < 0) data *= (-1);

	for (i = 0; ; i++, len++)
	{
		if ((data /= 10) <= 0)
			break;
	}
	return(len);
}

static void _wnoti_changed_cb(keynode_t * node, void *data)
{
	_D("%s", __func__);
	struct appdata *ad = (struct appdata *)data;
	ret_if(ad == NULL);

	int ret = -1;
	int count = -1;
	char buf[16] = { 0, };

	ret = vconf_get_int(WNOTI_COUNT, &count);
	if(ret < 0) {
		_E("Failed to get vconf value");
		return;
	}

	if(count == 0) {
		_D("count is zero");
		elm_object_signal_emit(ad->page_layout[ad->idle_clock_page + 1], "change,badge,hidden", "img.badge");
		elm_object_signal_emit(ad->page_layout[ad->idle_clock_page + 1], "hide,txt,badge", "txt.badge");
	} else {
		_D("count is %d", count);
		if(_get_length_of_count(count) == 1) {
			elm_object_signal_emit(ad->page_layout[ad->idle_clock_page + 1], "change,badge,1digit", "img.badge");
		} else if(_get_length_of_count(count) == 2) {
			elm_object_signal_emit(ad->page_layout[ad->idle_clock_page + 1], "change,badge,2digit", "img.badge");
		} else if(_get_length_of_count(count) == 3) {
			elm_object_signal_emit(ad->page_layout[ad->idle_clock_page + 1], "change,badge,3digit", "img.badge");
		}
		elm_object_signal_emit(ad->page_layout[ad->idle_clock_page + 1], "show,txt,badge", "txt.badge");
		snprintf(buf, sizeof(buf), "%d", count);
		elm_object_part_text_set(ad->page_layout[ad->idle_clock_page + 1], "txt.badge", buf);
	}
}

static char *_minictrl_tag_get(Evas_Object *obj)
{
	char *tag = NULL;

	if (obj != NULL) {
		tag = evas_object_data_get(obj, E_DATA_MINICONTROL_TAG);
	}

	return tag;
}

static void _minictrl_tag_del(Evas_Object *obj)
{
	char *tag = NULL;

	if (obj != NULL) {
		tag = evas_object_data_get(obj, E_DATA_MINICONTROL_TAG);
		if (tag != NULL) {
			free(tag);
		}
		evas_object_data_set(obj, E_DATA_MINICONTROL_TAG, NULL);
	}
}

static void _minictrl_tag_add(Evas_Object *obj, const char *tag)
{
	char *tag_new = NULL;

	if (obj != NULL && tag != NULL) {

		_minictrl_tag_del(obj);

		tag_new = strdup(tag);
		if (tag_new != NULL) {
			evas_object_data_set(obj, E_DATA_MINICONTROL_TAG, tag_new);
		} else {
			evas_object_data_set(obj, E_DATA_MINICONTROL_TAG, NULL);
		}
	}
}

static void _minictrl_viewer_deleted_cb(void *data, Evas *e, Evas_Object *obj, void *event_info)
{
	char *tag = NULL;

	if (obj != NULL) {
		tag = _minictrl_tag_get(obj);
		if (tag != NULL) {
			_E("tag_old:%s", tag);
		}

		_minictrl_tag_del(obj);
	}
}

void destroy_minictrl(void *data, const char *name)
{
	struct appdata *ad = data;
	ret_if(ad == NULL);
	ret_if(name == NULL);

	char *tag_old = NULL;
	Evas_Object *viewer = NULL;

	if (ad->minictl_idle_clock != NULL) {
		viewer = elm_object_part_content_get(ad->minictl_idle_clock, "sw.clock");
		tag_old = _minictrl_tag_get(viewer);

		_E("tag_old:%s deleted:%s", tag_old, name);

		if (tag_old != NULL && name != NULL) {
			if (strcmp(tag_old, name) == 0) {
				_D("minictrl delete:%s", name);
				_minictrl_tag_del(viewer);
				elm_object_part_content_unset(ad->minictl_idle_clock, "sw.clock");
				if(viewer) {
					evas_object_del(viewer);
				}
			}
		}
	} else {
		_E("failed to get idle clock container");
	}

#if 0
	if(ad->minictl_idle_clock) {
		evas_object_del(ad->minictl_idle_clock);
		ad->minictl_idle_clock = NULL;
	}
#endif
}

static void _minictrl_update(const char *name, unsigned int width, unsigned int height, void *data)
{
	struct appdata *ad = data;

	ret_if(name == NULL);
	ret_if(ad == NULL);

	_D("resized : w(%d) h(%d)", width, height);
	evas_object_resize(ad->minictl_idle_clock, width, height);
}

#define DEFAULT_MINICONTROL_NAME "idle-clock-digital"
#define APP_NAME_EMULATOR_DEFAULT_CLOCK "org.tizen.idle-clock-digital"

static void _minictrl_add(const char *name, unsigned int width, unsigned int height, minicontrol_priority_e priority, void *data)
{
	struct appdata *ad = data;
	Evas_Object *viewer = NULL;
	Evas_Object *viewer_old = NULL;
	char minictrl_name[255] = {0};
	char pkgname[255] = {0};
	int pid = 0;
	int i = 0;
	int ret_aul = 0;
	char *pkg_name = NULL;

	ret_if(name == NULL);
	ret_if(ad == NULL);

	_D("_minictrl_add() name=%s", name);

	viewer = minicontrol_viewer_add(ad->minictl_idle_clock, name);
	if(viewer == NULL) {
		_E("fail to add viewer - %s", name);
		return;
	}

	viewer_old = elm_object_part_content_get(ad->minictl_idle_clock, "sw.clock");
	if (viewer_old != NULL) {
		_minictrl_tag_del(viewer_old);
	}
	_minictrl_tag_add(viewer, name);
	evas_object_event_callback_add(viewer, EVAS_CALLBACK_DEL, _minictrl_viewer_deleted_cb, NULL);

	elm_object_part_content_set(ad->minictl_idle_clock, "sw.clock", viewer);
	_D("resized : w(%d) h(%d)", width, height);

	name++;
	while(*name != ']'){
		minictrl_name[i++] = *(name++);
	}
	minictrl_name[i] = NULL;
	_D("pkgname : %s", minictrl_name);

	if(strcmp(minictrl_name, DEFAULT_MINICONTROL_NAME)){
		_D("Web minictrl is added");
		pid = atoi(minictrl_name);
		ret_aul = aul_app_get_pkgname_bypid(pid, pkgname, sizeof(pkgname));
		if(ret_aul == AUL_R_ERROR){
			_E("no such pkgname");
			return;
		}
		_D("pkgname : %s", pkgname);
		pkg_name = get_app_pkgname(pkgname);
		_D("pkg_name : %s", pkg_name);
		vconf_set_str(VCONFKEY_WMS_CLOCKS_SET_IDLE, pkg_name);
		/*
		ad->appid_idle_clock_on_top = strdup(pkgname);
		_D("ad->appid_idle_clock_on_top : %s", ad->appid_idle_clock_on_top);
	}
	else{
		ad->appid_idle_clock_on_top = strdup(APP_NAME_EMULATOR_DEFAULT_CLOCK);
		_D("ad->appid_idle_clock_on_top : %s", ad->appid_idle_clock_on_top);
		*/
	}
}

static void _mctrl_monitor_cb(minicontrol_action_e action, const char *name, unsigned int width, unsigned int height, minicontrol_priority_e priority, void *data)
{
	if(data == NULL || name == NULL) {
		_D("data is NULL or name is NULL");
		return;
	}

	switch (action) {
	case MINICONTROL_ACTION_START:
		_D(" >>>> Start minictl");
		_minictrl_add(name, width, height, priority, data);
		break;
	case MINICONTROL_ACTION_RESIZE:
		_D(" >>>> Resize minictl");
		_minictrl_update(name, width, height, data);
		break;
	case MINICONTROL_ACTION_STOP:
		_D(" >>>> Destroy minictl");
		destroy_minictrl(data, name);
		break;
	case MINICONTROL_ACTION_REQUEST:
		_D("MINICONTROL_ACTION_REQUEST");
		break;
	default:
		break;
	}
}

#if USE_MAPBUF
static Evas_Object *_set_page_map_buf(Evas_Object *box, Evas_Object *layout)
{
	_D("%s", __func__);
	Evas_Object *mapbuf = NULL;

	mapbuf = elm_mapbuf_add(box);
#if 0
	elm_mapbuf_enabled_set(mapbuf, EINA_TRUE);
	elm_mapbuf_smooth_set(mapbuf, EINA_TRUE);
	elm_mapbuf_alpha_set(mapbuf, EINA_TRUE);
#endif
	//evas_object_size_hint_weight_set(mapbuf, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_weight_set(mapbuf, 0, 0);
	//evas_object_size_hint_align_set(mapbuf, EVAS_HINT_FILL, EVAS_HINT_FILL);
	evas_object_size_hint_align_set(mapbuf, 0, EVAS_HINT_FILL);
	elm_object_content_set(mapbuf, layout);
	evas_object_show(mapbuf);
	//elm_box_pack_end(box, mapbuf);

	return mapbuf;
}
#endif

static void _create_box_contents(Evas_Object *box, void *data)
{
	_D("%s", __func__);
	struct appdata *ad = (struct appdata *)data;
	ret_if(ad == NULL);
	ret_if(box == NULL);

	Evas_Object *layout = NULL;
	Evas_Object *icon = NULL;

	int ret = -1;
	int temp = 0;
	int i = 0;
	int page_count = 0;

#ifndef EMULATOR_TYPE
	/* Favorite apps related of Call(1 ~ 2) */
	if(get_app_label(get_app_id(PKG_NAME_LOGS)) == NULL) {
		_D("Failed to get favorite data : %s", PKG_NAME_LOGS);
	} else {
		ret = vconf_get_bool(VCONFKEY_WMS_SYNC_LOGS, &temp);
		if(ret < 0) {
			_D("Failed to get VCONFKEY_WMS_SYNC_LOGS");
		}
		if(temp == 1) {
			layout = elm_layout_add(box);
			elm_layout_file_set(layout, EDJ_APP, "scroller_box");
			icon = make_icon(ad, ad->ly_main, ICON_PATH_LOGS);
			elm_object_part_content_set(layout, "sw.icon", icon);

			elm_object_part_text_set(layout, "txt.app.name", get_app_label(get_app_id(PKG_NAME_LOGS)));
			evas_object_resize(layout, WIN_SIZE, WIN_SIZE);
			evas_object_show(layout);
			ad->page_layout[page_count] = layout;
			elm_object_signal_callback_add(layout, "clicked,rect,icon", "*", _launch_app_cb, PKG_NAME_LOGS);
			elm_object_signal_callback_add(layout, "clicked,rect,text", "*", _launch_app_cb, PKG_NAME_LOGS);
#if USE_MAPBUF
			// mapbuf for LOGS
			ad->mapbuf[page_count] = _set_page_map_buf(box, layout);
			elm_box_pack_end(box, ad->mapbuf[page_count]);

#else
			elm_box_pack_end(box, layout);
#endif
			page_count++;
		}
	}

	if(get_app_label(get_app_id(PKG_NAME_CONTACTS)) == NULL) {
		_D("Failed to get favorite data : %s", PKG_NAME_CONTACTS);
	} else {
		ret = vconf_get_bool(VCONFKEY_WMS_SYNC_CONTACTS, &temp);
		if(ret < 0) {
			_D("Failed to get VCONFKEY_WMS_SYNC_CONTACTS");
		}
		if(temp == 1) {
			layout = elm_layout_add(box);
			elm_layout_file_set(layout, EDJ_APP, "scroller_box");
			icon = make_icon(ad, ad->ly_main, ICON_PATH_CONTACTS);
			elm_object_part_content_set(layout, "sw.icon", icon);

			elm_object_part_text_set(layout, "txt.app.name", get_app_label(get_app_id(PKG_NAME_CONTACTS)));
			evas_object_resize(layout, WIN_SIZE, WIN_SIZE);
			evas_object_show(layout);
			ad->page_layout[page_count] = layout;
			elm_object_signal_callback_add(layout, "clicked,rect,icon", "*", _launch_app_cb, PKG_NAME_CONTACTS);

#if USE_MAPBUF
			// mapbuf for CONTACTS
			ad->mapbuf[page_count] = _set_page_map_buf(box, layout);
			elm_box_pack_end(box, ad->mapbuf[page_count]);
#else
			elm_box_pack_end(box, layout);
#endif
			page_count++;
		}
	}
#endif

	/* PAGE - clock */
	ad->idle_clock_page = page_count;
#if USE_SETTING_IDLE_CLOCK
	layout = elm_layout_add(box);
	elm_layout_file_set(layout, EDJ_APP, "idle_clock");
	evas_object_resize(layout, WIN_SIZE, WIN_SIZE);
	evas_object_show(layout);
	ad->page_layout[page_count] = layout;
	ad->minictl_idle_clock = layout;

	minicontrol_error_e m_ret = MINICONTROL_ERROR_NONE;
	m_ret = minicontrol_monitor_start(_mctrl_monitor_cb, ad);
	if (m_ret != MINICONTROL_ERROR_NONE) {
		_E("fail to minicontrol_monitor_start()- %d", m_ret);
	}
	_D(" >>>>>>> pack end minictl");
	elm_box_pack_end(box, layout);
#else
	int weather_id = -1;
	char *temp_unit = NULL;
	char buf[128] = { 0, };

	layout = elm_layout_add(box);
	//elm_layout_file_set(layout, EDJ_APP, "layout_clock_date");
	elm_layout_file_set(layout, EDJ_APP, "layout_clock_weather");

	/* weather - temp */
	ret = vconf_get_int(VCONFKEY_DAILYBRIEFING_WEATHER_TEMP, &temp);
	if(ret < 0) {
		_D("Failed to get temp");
	} else {
		_D("temperature = %d", temp);
		snprintf(buf, sizeof(buf), "%d", temp);
		elm_object_part_text_set(layout, "txt.temp", buf);
	}

	/* weather - celsius/fahrenheit */
	temp_unit = vconf_get_str(VCONFKEY_DAILYBRIEFING_WEATHER_TEMP_UNIT);
	if(temp_unit == NULL || strlen(temp_unit) < 1){
		_D("temp unit is NULL : use celsius");
		elm_object_signal_emit(layout, "change.weather.temp.celsius", "img.temp.unit");
	}else {
		_D("unit = %s", temp_unit);
		if(!strncmp(temp_unit, TEMP_UNIT_C, sizeof(TEMP_UNIT_C))) {
			elm_object_signal_emit(layout, "change.weather.temp.celsius", "img.temp.unit");
		} else if(!strncmp(temp_unit, TEMP_UNIT_F, sizeof(TEMP_UNIT_F))) {
			elm_object_signal_emit(layout, "change.weather.temp.fahrenheit", "img.temp.unit");
		}
		free(temp_unit);
		temp_unit = NULL;
	}

	/* weather - icon */
	ret = vconf_get_int(VCONFKEY_DAILYBRIEFING_WEATHER_ICON_ID, &weather_id);
	_D("weather icon id = %d", weather_id);
	if(ret < 0) {
		_D("weather icon id is NULL");
		icon = make_icon(ad, ad->ly_main, WEATHER_ICON_SUNNY);
		elm_object_part_content_set(layout, "sw.weather.icon", icon);
	} else {
		icon = make_icon(ad, ad->ly_main, weather_icon_path[weather_id]);
		elm_object_part_content_set(layout, "sw.weather.icon", icon);
	}

	/* register vconf changed cb */
	ret = vconf_notify_key_changed(VCONFKEY_DAILYBRIEFING_WEATHER_TEMP, _temp_changed_cb, ad);
	if(ret < 0) {
		_D("Failed to register vconf changed cb : VCONFKEY_DAILYBRIEFING_WEATHER_TEMP");
	}
	ret = vconf_notify_key_changed(VCONFKEY_DAILYBRIEFING_WEATHER_TEMP_UNIT, _temp_unit_changed_cb, ad);
	if(ret < 0) {
		_D("Failed to register vconf changed cb : VCONFKEY_DAILYBRIEFING_WEATHER_TEMP_UNIT");
	}
	ret = vconf_notify_key_changed(VCONFKEY_DAILYBRIEFING_WEATHER_ICON_ID, _temp_icon_changed_cb, ad);
	if(ret < 0) {
		_D("Failed to register vconf changed cb : VCONFKEY_DAILYBRIEFING_WEATHER_ICON_ID");
	}

	evas_object_resize(layout, WIN_SIZE, WIN_SIZE);
	evas_object_show(layout);
	ad->page_layout[page_count] = layout;
	set_info(ad);
#if USE_MAPBUF
	// mapbuf for clock
	ad->mapbuf[page_count] = _set_page_map_buf(box, layout);
#else
	elm_box_pack_end(box, layout);
#endif

#endif

	page_count++;

#ifndef EMULATOR_TYPE
	/* PAGE - notification */
	if(get_app_label(get_app_id(PKG_NAME_NOTIFICATION)) == NULL) {
		_D("Failed to get favorite data : %s", PKG_NAME_NOTIFICATION);
	} else {
		layout = elm_layout_add(box);
		elm_layout_file_set(layout, EDJ_APP, "scroller_box");
		icon = make_icon(ad, ad->ly_main, ICON_PATH_NOTIFICATION);
		elm_object_part_content_set(layout, "sw.icon", icon);

		elm_object_part_text_set(layout, "txt.app.name", get_app_label(get_app_id(PKG_NAME_NOTIFICATION)));
		evas_object_resize(layout, WIN_SIZE, WIN_SIZE);
		evas_object_show(layout);
		ad->page_layout[page_count] = layout;
		elm_object_signal_callback_add(layout, "clicked,rect,icon", "*", _launch_app_cb, PKG_NAME_NOTIFICATION);

		/* register vconf changed cb */
		ret = vconf_notify_key_changed(WNOTI_COUNT, _wnoti_changed_cb, ad);
		if(ret < 0) {
			_D("Failed to register vconf changed cb : VCONFKEY_DAILYBRIEFING_WEATHER_TEMP");
		}
		_wnoti_changed_cb(NULL, ad);

#if USE_MAPBUF
		// mapbuf for notification
		ad->mapbuf[page_count] = _set_page_map_buf(box, layout);
		elm_box_pack_end(box, ad->mapbuf[page_count]);
#else
		elm_box_pack_end(box, layout);
#endif
		page_count++;
	}

	/* favorite apps */
	favorite_apps_info *info = NULL;
	if(ad->favorite_apps_list->len == 0) {
		_D("The number of favorite apps is zero : %d", ad->favorite_apps_list->len);
	} else {
		for(i = 0; i < ad->favorite_apps_list->len; i++) {
			info = g_array_index(ad->favorite_apps_list, favorite_apps_info*, i);

			_D("pkgname : %s / order : %d", info->pkgname, info->order);
			if(info->pkgname == NULL || get_app_label(get_app_id(info->pkgname)) == NULL) {
				_D("[%d]Failed to get favorite data : pkgname", i);
				continue;
			}

			layout = elm_layout_add(box);
			elm_layout_file_set(layout, EDJ_APP, "scroller_box");
			icon = make_icon(ad, ad->ly_main, get_app_icon(get_app_id(info->pkgname)));
			elm_object_part_content_set(layout, "sw.icon", icon);

			elm_object_part_text_set(layout, "txt.app.name", get_app_label(get_app_id(info->pkgname)));
			evas_object_resize(layout, WIN_SIZE, WIN_SIZE);
			evas_object_show(layout);
			ad->page_layout[i+page_count] = layout;
			elm_object_signal_callback_add(layout, "clicked,rect,icon", "*", _launch_app_cb, get_app_id(info->pkgname));
			elm_object_signal_callback_add(layout, "clicked,rect,text", "*", _launch_app_cb, get_app_id(info->pkgname));
#if USE_MAPBUF
			// mapbuf for favorite apps
			ad->mapbuf[i+page_count] = _set_page_map_buf(box, layout);
			elm_box_pack_end(box, ad->mapbuf[i+page_count]);
#else
			elm_box_pack_end(box, layout);
#endif
		}
		page_count += i;
		memset(info, '\0', sizeof(favorite_apps_info));
	}
#endif

	/* PAGE - more apps */
	layout = elm_layout_add(box);
	elm_layout_file_set(layout, EDJ_APP, "scroller_box");
	icon = make_icon(ad, ad->ly_main, ICON_PATH_MORE_APPS);
	elm_object_part_content_set(layout, "sw.icon", icon);
	elm_object_signal_callback_add(layout, "clicked,rect,icon", "*", moreapps_click_cb, ad);

	elm_object_part_text_set(layout, "txt.app.name", _("IDS_IDLE_BODY_APPS"));
	elm_object_signal_callback_add(layout, "clicked,rect,text", "*", moreapps_click_cb, ad);
	evas_object_resize(layout, WIN_SIZE, WIN_SIZE);
	evas_object_show(layout);
	//ad->page_layout[(ad->favorite_apps_count/3 + 1)] = layout;
	ad->page_layout[page_count] = layout;

#if USE_MAPBUF
	// mapbuf for more apps
	ad->mapbuf[page_count] = _set_page_map_buf(box, layout);
	elm_box_pack_end(box, ad->mapbuf[page_count]);
#else
	elm_box_pack_end(box, layout);
#endif
}

static void _page_show(void *data, Evas * e, Evas_Object * obj, void *event_info)
{
	_D("%s", __func__);
	struct appdata *ad = (struct appdata *)data;

	elm_scroller_page_show(obj, ad->idle_clock_page, 0);
}

Evas_Object *create_main_scroller(Evas_Object *parent, void *data)
{
	_D("%s", __func__);
	struct appdata *ad = (struct appdata *)data;
	retv_if(ad == NULL, NULL);
	retv_if(parent == NULL, NULL);

	Evas_Object *scroller = NULL;
	Evas_Object *box = NULL;

	scroller = elm_scroller_add(parent);
	if(scroller == NULL) {
		_E("elm_scroller_add() failed");
		return NULL;
	}
	elm_scroller_loop_set(scroller, EINA_TRUE, EINA_FALSE);
	elm_object_theme_set(scroller, ad->theme);
	elm_object_style_set(scroller, "w-launcher-app/scrollbar");

	evas_object_size_hint_weight_set(scroller, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_align_set(scroller, EVAS_HINT_FILL, EVAS_HINT_FILL);
	//@TODO: need to check
	//elm_scroller_page_relative_set(scroller, 1.0, 0.0);

	elm_scroller_bounce_set(scroller, EINA_TRUE, EINA_FALSE);
	elm_scroller_policy_set(scroller,ELM_SCROLLER_POLICY_AUTO, ELM_SCROLLER_POLICY_AUTO);
	elm_scroller_page_scroll_limit_set(scroller, PAGE_MOVE, PAGE_MOVE);
	elm_scroller_content_min_limit(scroller, EINA_FALSE, EINA_TRUE);

	elm_object_scroll_lock_y_set(scroller, EINA_TRUE);
	elm_object_tree_focus_allow_set(scroller, EINA_FALSE);
	evas_object_show(scroller);

	elm_scroller_page_size_set(scroller, WIN_SIZE, WIN_SIZE);

	box = elm_box_add(scroller);
	if(box == NULL) {
		_E("Failed to add elm box");
		evas_object_del(scroller);
		return NULL;
	}
	evas_object_size_hint_weight_set(box, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_size_hint_fill_set(box, EVAS_HINT_FILL, EVAS_HINT_FILL);
	elm_box_horizontal_set(box, EINA_TRUE);
	evas_object_show(box);
	elm_box_layout_set(box, _box_layout_cb, ad, NULL);
	evas_object_data_set(box, "SCROLLER", scroller);
	_create_box_contents(box, ad);
	elm_object_content_set(scroller, box);

	ad->main_box = box;

	Eina_List *list = elm_box_children_get(box);
	if(eina_list_count(list) <= 1)
		elm_scroller_loop_set(scroller, EINA_FALSE, EINA_FALSE);

#if 0
	evas_object_smart_callback_add(scroller, "scroll,anim,start", _scroller_anim_start_cb, ad);
	evas_object_smart_callback_add(scroller, "scroll,drag,start", _scroller_drag_start_cb, NULL);
	evas_object_smart_callback_add(scroller, "scroll,drag,stop", _scroller_drag_stop_cb, NULL);
#endif
	evas_object_smart_callback_add(scroller, "scroll,anim,stop", _scroller_anim_stop_cb, ad);

	/* set first page : idle clock */
	evas_object_event_callback_add(scroller, EVAS_CALLBACK_RESIZE, _page_show, ad);

	return scroller;
}
