/*
 * Copyright 2012  Samsung Electronics Co., Ltd
 *
 * Licensed under the Flora License, Version 1.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.tizenopensource.org/license
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <Ecore.h>
#include <Ecore_X.h>

#include "main.h"
#include "log.h"

Ecore_X_Window _add_input_window(void)
{
	Ecore_X_Window win = 0;
	win = ecore_x_window_input_new(0, -10000, -10000, 1, 1);
	if(win){
		//ecore_x_event_mask_unset(win, ECORE_X_EVENT_MASK_NONE);
		ecore_x_icccm_title_set(win, "w-launcher-app-input-window");
		ecore_x_netwm_name_set(win, "w-launcher-app-input-window");
		ecore_x_netwm_pid_set(win, getpid());
		ecore_x_flush();
		return win;
	}
	else{
		_E("Failed to add input window");
		return 0;
	}
}

Evas_Object *_add_window(const char *name, Eina_Bool _is_main)
{
	Evas_Object *eo = NULL;
	int w = -1, h = -1;

	/* Open GL backedn */
	if(_is_main)
		elm_config_preferred_engine_set("opengl_x11");

	eo = elm_win_add(NULL, name, ELM_WIN_BASIC);
	if (eo) {
		elm_win_title_set(eo, name);
		elm_win_borderless_set(eo, EINA_TRUE);
		elm_win_alpha_set(eo, EINA_FALSE);
		ecore_x_window_size_get(ecore_x_window_root_first_get(), &w, &h);
		if(w == -1 || h == -1){
			_E("Failed to get x_window size");
			return NULL;
		}
		evas_object_resize(eo, w, h);
	}
	return eo;
}

//BG create function

Evas_Object* _create_bg(Evas_Object *parent)
{
	Evas_Object *bg;
	if (parent == NULL) return NULL;
	bg = elm_bg_add(parent);
	evas_object_size_hint_weight_set(bg, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	elm_win_resize_object_add(parent, bg);
	evas_object_show(bg);
	return bg;
}


Evas_Object *_add_layout(Evas_Object *parent, const char *file,
			     const char *group)
{
	_D("%s", __func__);
	Evas_Object *eo = NULL;
	int r = -1;

	retvm_if(parent == NULL, NULL, "Invalid argument: parent is NULL\n");
	retvm_if(file == NULL, NULL, "Invalid argument: file is NULL\n");
	retvm_if(group == NULL, NULL, "Invalid argument: group is NULL\n");

	eo = elm_layout_add(parent);
	retvm_if(eo == NULL, NULL, "Failed to add layout\n");

	r = elm_layout_file_set(eo, file, group);
	if (!r) {
		_E("Failed to set file[%s]\n", file);
		evas_object_del(eo);
		return NULL;
	}

	evas_object_size_hint_weight_set(eo, EVAS_HINT_EXPAND, EVAS_HINT_EXPAND);
	evas_object_show(eo);
	return eo;
}
